//
//  AGHomeFolderSize.m
//  AGTrashStatusMenu
//
//	Copyright (C) 2007 Adam Gerson
//
//	This program is free software; you can redistribute it and/or
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#import "AGHomeFolderSize.h"

@implementation AGHomeFolderSize

- (id)init
{	
	[super init];
	[self updateDiskSpaceInfo];
	return self;
}

-(void)dealloc 
{ 
	[super dealloc]; 
} 

- (void)updateDiskSpaceInfo
{
        diskUse=0; diskFree=0; diskSize=0;
        hardUse=0; hardFree=0; hardSize=0;
        softFree=0; softSize=0; softOver=0; softTime=0;
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString *pathToHomeFolder = [@"~" stringByExpandingTildeInPath];
	NSDictionary *homeFolderFileSystemAttributes = 
		[fileManager fileSystemAttributesAtPath:pathToHomeFolder];

	diskSize = [[homeFolderFileSystemAttributes objectForKey:NSFileSystemSize] longLongValue];
	diskFree =[[homeFolderFileSystemAttributes objectForKey:NSFileSystemFreeSize] longLongValue];
	diskUse  = diskSize-diskFree;

//NSLog(@"diskSize=%qi diskFree=%qi diskUse=%qi", diskSize, diskFree, diskUse);

	struct dqblk dq;
	int uid=getuid();
	if (quotactl([pathToHomeFolder cString], QCMD(Q_GETQUOTA,USRQUOTA), uid, (char *) &dq) == 0) {
		hardSize = dq.dqb_bhardlimit;
		hardUse  = dq.dqb_curbytes;
		hardFree = hardSize-hardUse;
		softSize = dq.dqb_bsoftlimit;
		softFree = softSize-hardUse;
		softTime = dq.dqb_btime;
		softOver = (hardUse>softSize)?(hardUse-softSize):0;

//NSLog(@"hardSize=%qi hardFree=%qi hardUse=%qi", hardSize, hardFree, hardUse);
//NSLog(@"softSize=%qi softFree=%qi softOver=%qi softTime=%qi", softSize, softFree, softOver, softTime);
        }

	
	[fileManager release];
	return;
}

- (NSString *)humanReadableSize:(long long)bytes
{
	NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
	[numberFormatter setFormatterBehavior:NSNumberFormatterBehavior10_4];
	[numberFormatter setFormat:@"0.##"];

	static NSString *suffix[] = { @"B", @"KB", @"MB", @"GB", @"TB", @"PB", @"EB" };
	int i, c = 7;
	double size = bytes;

	// note that if soft quotas are in place, free space might be negative, hence the fabs()
	for (i = 0; i < c && fabs(size) >= 1024; i++)
	{
		size = size / 1024;
	}
	NSString *formattedNumber = [numberFormatter stringFromNumber:[NSNumber numberWithDouble:size]];
	return [NSString stringWithFormat:@"%@ %@",formattedNumber, suffix[i]];
}

- (NSString *)humanReadableTime:(int)seconds
{
  if (seconds> 86400) {
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
    [numberFormatter setFormatterBehavior:NSNumberFormatterBehavior10_4];
    [numberFormatter setFormat:@"0.##"];

    double days=seconds/86400.0;

    NSString *formattedNumber = [numberFormatter stringFromNumber:[NSNumber numberWithDouble:days]];

    return [NSString stringWithFormat:@"%@ days", formattedNumber];
  } else {
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
    [numberFormatter setFormatterBehavior:NSNumberFormatterBehavior10_4];
    [numberFormatter setFormat:@"00"];
    int secs = seconds%60;
    int mins = ((seconds-secs)/60)%60;
    int hours = (seconds-mins*60-secs)/3600;
    NSString *formattedSecs = [numberFormatter stringFromNumber:[NSNumber numberWithInt:secs]];
    NSString *formattedMins = [numberFormatter stringFromNumber:[NSNumber numberWithInt:mins]];
    NSString *formattedHours = [numberFormatter stringFromNumber:[NSNumber numberWithInt:hours]];
    return [NSString stringWithFormat:@"%@:%@:%@", formattedHours, formattedMins, formattedSecs];
  }
}

- (long long) getDiskSize { return diskSize; }
- (long long) getDiskUse  { return diskUse; }
- (long long) getDiskFree { return diskFree; }
- (long long) getHardSize { return hardSize; }
- (long long) getHardUse  { return hardUse; }
- (long long) getHardFree { return hardFree; }
- (long long) getSoftSize { return softSize; }
- (long long) getSoftFree { return softFree; }
- (long long) getSoftOver { return softOver; }
- (long long) getSoftTime { return softTime; }


@end
