//
//  AGTrashStatusItemController.m
//  AGTrashStatusMenu
//
//	Copyright (C) 2007 Adam Gerson
//
//	This program is free software; you can redistribute it and/or
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <Security/Security.h>

#import "AGTrashStatusItemController.h"

#define REFRESH_RATE_SECONDS 11.0
#define ALARM_PERCENT 95
#define WARN_PERCENT 90

@implementation AGTrashStatusItemController
- init
{
    if ((self = [super init]) != NULL) {
        whatIsThisText=nil;
	alertIsUp=0;
	alertTime=0;
    }
    return self;
}

-(void)dealloc 
{ 
	[statusItem release];
	[updateTimer release];
	[super dealloc]; 
} 

- (void)awakeFromNib 
{ 
	NSMutableDictionary *defaultValues;

	[self readInSettings];
	homeFolderSize = [[AGHomeFolderSize alloc] init];  
	
	
        defaultValues = [NSMutableDictionary dictionary];

        [defaultValues setObject: [NSNumber numberWithFloat:
                 [[settings objectForKey:@"redPercent"] floatValue]]
                                   forKey: redLimitPrefName];       
        [defaultValues setObject: [NSNumber numberWithFloat:
                 [[settings objectForKey:@"alertPercent"] floatValue]]
                                   forKey: alertLimitPrefName];       
        [defaultValues setObject: [NSNumber numberWithInt: 0]
                                   forKey: redLimitTypePrefName];       
        [defaultValues setObject: [NSNumber numberWithInt: 0]
                                   forKey: alertLimitTypePrefName];       
        [defaultValues setObject: [NSNumber numberWithInt:
                 [[settings objectForKey:@"updateMinutes"] intValue]]
                                   forKey: updateMinutesPrefName];       
        [defaultValues setObject: [NSNumber numberWithInt:
                 [[settings objectForKey:@"alertMinutes"] intValue]]
                                   forKey: alertMinutesPrefName];       

	[[NSUserDefaults standardUserDefaults] registerDefaults: defaultValues];

	//NSLog ([[settings objectForKey:@"xx"] stringValue]);
	
	//Load preferences
	[self readInUserPrefs];

	[self createStatusItem];
}

- (void)readInUserPrefs
{
    NSUserDefaults *defaults;
    defaults = [NSUserDefaults standardUserDefaults];

    redLimit = [defaults floatForKey: redLimitPrefName];
    redLimitType = [defaults floatForKey: redLimitTypePrefName];
    alertLimit = [defaults floatForKey: alertLimitPrefName];
    alertLimitType = [defaults floatForKey: alertLimitTypePrefName];
    updateMinutes = [defaults floatForKey: updateMinutesPrefName];
    alertMinutes = [defaults floatForKey: alertMinutesPrefName];

    //this line only matters when debugging, and checking disk every second
    if (alertMinutes < updateMinutes) alertMinutes=updateMinutes;
    //updateMinutes can be set to zero (which really becomes 30 seconds),
    //but alertMinutes can not be.
    if (alertMinutes < 1) alertMinutes=1;
}

- (void)readInSettings 
{ 
	//settings = [NSDictionary dictionaryWithObjectsAndKeys:
//			[NSNumber numberWithBool:YES],@"rfrr",
//	nil];
//			
//			[settings writeToFile:@"settings.plist" atomically:YES];
//			
			
			
NSBundle *thisBundle = [NSBundle bundleForClass:[self class]];
	NSString *path = [thisBundle pathForResource:@"settings" ofType:@"plist"];
	settings = [[NSDictionary alloc] initWithContentsOfFile:path];

	NSString *tmp = [thisBundle pathForResource:@"homeQuota" ofType:@"tiff"]; 
	blackIcon = [[NSImage alloc] initWithContentsOfFile:tmp];

	tmp = [thisBundle pathForResource:@"homeQuotaRed" ofType:@"tiff"]; 
	redIcon = [[NSImage alloc] initWithContentsOfFile:tmp];
}

- (void)menuWillOpen:(NSMenu *) mn
{
	//NSLog(@"menuWillOpen!");
	if (alertIsUp || emptyIsUp || explainIsUp) { [self bringAppToFront]; }
}

- (void)createStatusItem 
{ 
	statusItem = [[[NSStatusBar systemStatusBar] 
		statusItemWithLength:NSVariableStatusItemLength] retain];
	[statusItem setHighlightMode:YES]; 
	[statusItem setEnabled:YES]; 
	[statusItem setToolTip:@"Home Directory Disk Space Monitor"];
	[statusItem setMenu:theMenu]; 
	
	[self updateMenu];
	
	double refreshRateInSeconds = updateMinutes*60.0;
	if (refreshRateInSeconds < 30) { refreshRateInSeconds = 30.0; }
	//For testing:
	//refreshRateInSeconds = 1.0;
	updateTimer = [[NSTimer 
               scheduledTimerWithTimeInterval:refreshRateInSeconds
               target:self 
               selector:@selector(updateMenu:) 
               userInfo:nil 
               repeats:YES] retain]; 
	[updateTimer fire]; 
}

- (void)updateMenu:(NSTimer *) timer
{ 
	[self updateMenu];
}

- (void)updateMenu
{ 
	NSColor *iconTextColor;
	NSImage *statusItemIcon;
        int doRed=0;
        int doAlert=0;
	int percentUsed;
        long long diskUse, diskSize, diskFree;
        long long hardUse, hardSize, hardFree;
        long long          softSize, softFree;
        long long softOver, softTime;


//NSLog(@"Updating...");
	[homeFolderSize updateDiskSpaceInfo];
	diskUse  = [homeFolderSize getDiskUse];
	diskSize = [homeFolderSize getDiskSize];
	diskFree = [homeFolderSize getDiskFree];
	hardUse  = [homeFolderSize getHardUse];
	hardSize = [homeFolderSize getHardSize];
	hardFree = [homeFolderSize getHardFree];
	softSize = [homeFolderSize getSoftSize];
	softFree = [homeFolderSize getSoftFree];
	softOver = [homeFolderSize getSoftOver];
	softTime = [homeFolderSize getSoftTime];

        //clear top of menu
        while (! [[theMenu itemAtIndex:0] isSeparatorItem]) {
          [theMenu removeItemAtIndex:0];
        }

        currentMenuAddIndex=0;
	
        if (softSize || hardSize) {  // if has quota
          if (softSize && softSize<hardSize) {  // has soft quota
            percentUsed = (int)(100.0*hardUse/softSize);
	    whatIsThisText = [settings objectForKey:@"whatIsThisSoftText"];
            if (hardUse >= [self maxUse:redLimit type:redLimitType outOf:softSize]) {
              doRed=1;
            }
            if (hardUse >= [self maxUse:alertLimit type:alertLimitType outOf:softSize]) {
              doAlert=1;
            }

            if (softOver) {
              [self addItemToMenu: @"••• Soft Quota Exceeded by "
                       plusString: [[homeFolderSize humanReadableSize:softOver]
                                     stringByAppendingString:@" •••"]
              ];
              [self addItemToMenu: @"Time Remaining "
                     plusString: [homeFolderSize humanReadableTime:softTime]];
              [self addItemToMenu: @"Hard Quota Remaining "
                     plusString: [homeFolderSize humanReadableSize:hardFree]];
              [self addItemToMenu: @"Space Used "
                     plusString: [homeFolderSize humanReadableSize:hardUse]];
              [self addItemToMenu: @"Soft Quota Limit "
                     plusString: [homeFolderSize humanReadableSize:softSize]];
              [self addItemToMenu: @"Hard Quota Limit "
                     plusString: [homeFolderSize humanReadableSize:hardSize]];
            } else {
              [self addItemToMenu: @"Soft Quota Remaining "
                     plusString: [homeFolderSize humanReadableSize:softFree]];
              [self addItemToMenu: @"Soft Quota Used "
                     plusString: [homeFolderSize humanReadableSize:hardUse]];
              [self addItemToMenu: @"Soft Quota Limit "
                     plusString: [homeFolderSize humanReadableSize:softSize]];
              [self addItemToMenu: @"Hard Quota Limit "
                     plusString: [homeFolderSize humanReadableSize:hardSize]];
            }
            if (diskFree < softFree) {
              [self addItemToMenu: @"••• Disk Will Fill Before Soft Quota •••"];
              [self addItemToMenu: @"Disk Space Remaining "
                     plusString: [homeFolderSize humanReadableSize:diskFree]];
            } else if (diskFree < hardFree) {
              [self addItemToMenu: @"••• Disk Will Fill Before Hard Quota •••"];
              [self addItemToMenu: @"Disk Space Remaining "
                     plusString: [homeFolderSize humanReadableSize:diskFree]];
            }
	  } else {  // only hard quota
            percentUsed = (int)(100.0*hardUse/hardSize);
	    whatIsThisText = [settings objectForKey:@"whatIsThisHardText"];
            if (hardUse >= [self maxUse:redLimit type:redLimitType outOf:hardSize]) {
              doRed=1;
            }
            if (hardUse >= [self maxUse:alertLimit type:alertLimitType outOf:hardSize]) {
              doAlert=1;
            }
            [self addItemToMenu: @"Quota Remaining "
                   plusString: [homeFolderSize humanReadableSize:hardFree]];
            [self addItemToMenu: @"Quota Used "
                   plusString: [homeFolderSize humanReadableSize:hardUse]];
            [self addItemToMenu: @"Quota Limit "
                   plusString: [homeFolderSize humanReadableSize:hardSize]];
            if (diskFree < hardFree) {
              [self addItemToMenu: @"••• Disk Will Fill Before Quota •••"];
              [self addItemToMenu: @"Disk Space Remaining "
                     plusString: [homeFolderSize humanReadableSize:diskFree]];
            }
          }
	} else { // no quota, just report disk usage
          percentUsed = (int)(100.0*diskUse/diskSize);
          whatIsThisText = [settings objectForKey:@"whatIsThisText"];
          if (diskUse >= [self maxUse:redLimit type:redLimitType outOf:diskSize]) {
            doRed=1;
          }
          if (diskUse >= [self maxUse:alertLimit type:alertLimitType outOf:diskSize]) {
            doAlert=1;
          }
          //[self addItemToMenu: @"No Quotas Found"];
          [self addItemToMenu: @"Disk Space Remaining "
                 plusString: [homeFolderSize humanReadableSize:diskFree]];
          [self addItemToMenu: @"Disk Space Used "
                 plusString: [homeFolderSize humanReadableSize:diskUse]];
          [self addItemToMenu: @"Total Disk Space "
                 plusString: [homeFolderSize humanReadableSize:diskSize]];
	}

        if (doRed) {
		iconTextColor = [NSColor whiteColor];
		statusItemIcon = [redIcon copy];
        } else {
		iconTextColor = [NSColor whiteColor];
		statusItemIcon = [blackIcon copy];
        }

	NSString *iconText = [[NSNumber numberWithInt:(percentUsed)] stringValue];
	[statusItemIcon lockFocus];
	if ([iconText length] > 2) {
		iconTextColor = [NSColor blackColor];
		NSDictionary *iconAttrs =
		    [NSDictionary dictionaryWithObject:iconTextColor
		    forKey:NSForegroundColorAttributeName];

	    [iconText drawAtPoint:NSMakePoint(-1,-2) withAttributes:iconAttrs];
	} else if ([iconText length] == 1) {
		NSDictionary *iconAttrs =
		    [NSDictionary dictionaryWithObject:iconTextColor
		    forKey:NSForegroundColorAttributeName];

	    [iconText drawAtPoint:NSMakePoint(9,-2) withAttributes:iconAttrs];
	} else {  //2 digit
		NSDictionary *iconAttrs =
		    [NSDictionary dictionaryWithObject:iconTextColor
		    forKey:NSForegroundColorAttributeName];

	    [iconText drawAtPoint:NSMakePoint(3,-2) withAttributes:iconAttrs];
	}
	[statusItemIcon unlockFocus];
    [statusItem setImage:statusItemIcon];

    if (doAlert) {
      [self alertUser];
    } else {
      //if not over alert limit, set time to zero.  This lets notifications
      //happen immediately, if space was over, then under, then over.
      //Yes, this is supposed to be a feature.
      alertTime=0;
    }
}

- (long long)maxUse:(double)limit type:(int)type outOf:(long long)size
{
  if (type == 0) {  //percentage
    return (size - ((limit/100.0)*size));
  } else if (type == 1) {  //kilobytes
    return (size - (limit*1024));
  } else if (type == 2) {  //megabytes
    return (size - (limit*1024*1024));
  } else {
    return(0);
  }
}

- (void)addItemToMenu:(NSString *)menuText plusString:(NSString *)plus
{
  [[theMenu insertItemWithTitle:[menuText stringByAppendingString:plus]
                         action:nil
                  keyEquivalent:@""
                        atIndex:currentMenuAddIndex
   ] setEnabled:NO];
  ++currentMenuAddIndex;
}

- (void)addItemToMenu:(NSString *)menuText
{
  [self addItemToMenu:menuText plusString:@""];
}

- (void)alertUser 
{
	[self bringAppToFront];
        //ideally, if alert is up we'd also move alert to active Space
        if (alertIsUp) return;
	NSTimeInterval now=[NSDate timeIntervalSinceReferenceDate];
	if (now-alertTime < alertMinutes*60) return;

        alertIsUp=1;
	NSString *alertText = [settings objectForKey:@"alertText"];
	NSAlert *alert = [[NSAlert alloc] init];
	[alert addButtonWithTitle:@"Ok"];
	[alert addButtonWithTitle:@"Empty Trash"];
	[alert setMessageText:@"You are almost out of disk space to save files"];
	[alert setInformativeText:alertText];
	[alert setAlertStyle:NSWarningAlertStyle];
	int userClickedoOnButton = [alert runModal];
	if (userClickedoOnButton == NSAlertSecondButtonReturn) {
		[self emptyTrash:nil];
	}
	[alert release];
	alertIsUp=0;
	alertTime = [NSDate timeIntervalSinceReferenceDate];
} 

- (IBAction)prefsChanged:(id)sender
{
//NSLog(@"Prefs Changed!!!");
    [self readInUserPrefs];
    double refreshRateInSeconds = updateMinutes*60.0;
    if (refreshRateInSeconds < 30) { refreshRateInSeconds = 30.0; }
    //For testing:
    //refreshRateInSeconds = 1.0;
//NSLog(@"refresh=%f",refreshRateInSeconds);
    [updateTimer invalidate]; 
    [updateTimer release]; 
    updateTimer = [[NSTimer 
        scheduledTimerWithTimeInterval:refreshRateInSeconds
        target:self 
        selector:@selector(updateMenu:) 
        userInfo:nil 
        repeats:YES] retain]; 
    [updateTimer fire]; 
}

-(IBAction)explainToUser:(id)sender 
{
	[self bringAppToFront];
	NSAlert *alert = [[NSAlert alloc] init];
	[alert addButtonWithTitle:@"Ok"];
	[alert setMessageText:@"Disk Space Monitor"];
	[alert setInformativeText:whatIsThisText];
	[alert setAlertStyle:NSWarningAlertStyle];
	// This doesn't work for NSAlert class:
	//[[alert window] setlevel:NSFloatingWindowLevel];
	explainIsUp=1;
	[alert runModal];
	[alert release];
	explainIsUp=0;
} 


-(IBAction)emptyTrash:(id)sender 
{ 
	NSDictionary* errorDict;
    NSAppleEventDescriptor* returnDescriptor = NULL;
	NSAppleScript* scriptObject = [[NSAppleScript alloc] initWithSource:
		@"\
		tell app \"Finder\"\n\
		empty the trash\n\
		end tell\n\
	"];
	[self bringAppToFront];
	emptyIsUp=1;
	NSAlert *alert = [[NSAlert alloc] init];
	[alert addButtonWithTitle:@"Ok"];
	[alert addButtonWithTitle:@"Cancel"];
	[alert setMessageText:@"Are you sure you want to permanently remove the items in the trash?"];
	[alert setInformativeText:@"You can not undo this operation."];
	[alert setAlertStyle:NSWarningAlertStyle];
	if ([alert runModal] == NSAlertFirstButtonReturn) {
		returnDescriptor = [scriptObject executeAndReturnError: &errorDict];
	}
	[alert release];
	
    [scriptObject release];
	emptyIsUp=0;
	[self updateMenu];
} 

-(void)bringAppToFront
{ 
	[NSApp activateIgnoringOtherApps: YES];
} 

-(IBAction)quit:(id)sender 
{ 
	bool requireAdminToQuit = [[settings objectForKey:@"requireAdminToQuit"] boolValue];
	if (requireAdminToQuit) {
		if (![self askForAdminPassword]) {
			NSAlert *alert = [[NSAlert alloc] init];
			[alert addButtonWithTitle:@"Ok"];
			[alert setMessageText:@"Can not quit"];
			[alert setInformativeText:@"You must be an administrator to quit this program."];
			[alert setAlertStyle:NSWarningAlertStyle];
			[alert runModal];
			[alert release];
		}
		else {
			[NSApp terminate:self];
		}
	}
	else {
		[NSApp terminate:self];
	}
} 

-(BOOL)askForAdminPassword
{
	OSStatus err;
    static AuthorizationRef gAuthorization = NULL;
	AuthorizationCreate(NULL, NULL, 0, &gAuthorization);

	static const AuthorizationFlags  kFlags = 
                  kAuthorizationFlagInteractionAllowed 
                | kAuthorizationFlagExtendRights;
    AuthorizationItem   kActionRight = { "a", 0, 0, 0 };
    AuthorizationRights kRights      = { 1, &kActionRight };

    assert(gAuthorization != NULL);


    // Request the application-specific right.

        err = AuthorizationCopyRights(
            gAuthorization,         // authorization
            &kRights,               // rights
            NULL,                   // environment
            kFlags,                 // flags
            NULL                    // authorizedRights
        );
    

    if (err == noErr) {
       return YES;
    }
	else {
		return NO;
    }
	return NO;
}


@end
